@@ -144,7 +144,7 @@ class LupusecAPI:
                 return clean_content
 
         except aiohttp.client_exceptions.ClientConnectorError:
-            _LOGGER.error("Cannot connect to: ", url)
+            _LOGGER.error("Cannot connect to: %s", url)
             return {}
 
         except aiohttp.ContentTypeError:
@@ -339,6 +339,52 @@ class LupusecAPI:
         _LOGGER.debug("__init__.py.async_set_switch() finished.")
 
 
+    async def control_cover_async(self, zone, command) -> None:
+        """ Async method to control a single cover. """
+        """ zone corresponds to the Lupusec Zone Number of the cover"""
+        """ Expects: control = "on", "off", "stop" or level """
+        _LOGGER.debug("__init__.py.control_cover_async() called: ")
+        _LOGGER.info("...for cover (zone): %s, set mode: %s", zone, command)
+        execution = "a=1&z=" + str(zone) + "&shutter=" + command
+        # example: exec: 'a=1&z=3&shutter=on'
+        _LOGGER.debug("{ exec: %s}", execution)        
+        params = {"exec": execution}
+        # Control Cover
+        async with aiohttp.ClientSession(auth=self._auth) as session:
+            _LOGGER.debug("auth.encode: %s", self._auth.encode())            
+
+            # Get Session Token
+            _LOGGER.debug("__init__.py.control_cover_async(): REQUEST=%s", CONST.TOKEN_REQUEST)
+            token_response = await LupusecAPI.async_get_token(self, session)
+            _LOGGER.debug("async_get_token(): done. check response...")
+            print(token_response)
+            if (token_response != 0):
+                _LOGGER.debug("Token: %s", self._token)
+                headers = {"X-Token": self._token}
+                print(headers)
+
+                # SET_SWITCH
+                _LOGGER.debug("__init__.py.control_cover_async(): REQUEST=%s", CONST.EXECUTE_REQUEST)
+                control_cover_response = await LupusecAPI._async_api_post(self._ip_address, session, 
+                    CONST.EXECUTE_REQUEST, headers, params)
+                _LOGGER.debug("_async_api_post(): done. check response...")
+
+                if (sys.getsizeof(control_cover_response) > 0):
+                    _LOGGER.debug("RESULT_RESPONSE: %s", control_cover_response[CONST.RESPONSE_RESULT]) 
+                    if (control_cover_response[CONST.RESPONSE_RESULT] == 1):
+                        _LOGGER.debug("RESPONSE_MESSAGE: %s", control_cover_response[CONST.RESPONSE_MESSAGE]) 
+                        if (len(control_cover_response[CONST.RESPONSE_MESSAGE]) != 0):
+                            _LOGGER.info("...cover (zone): %s, command: %s", zone, command)
+                    else :
+                        _LOGGER.info("ERROR: RESULT_RESPONSE: %s", control_cover_response[CONST.RESPONSE_RESULT])
+                        _LOGGER.info("RESPONSE_MESSAGE: %s", control_cover_response[CONST.RESPONSE_MESSAGE])  
+            else :    
+                _LOGGER.info("ERROR: no session Token available.")
+            
+        _LOGGER.debug("__init__.py.control_cover_async() finished.")
+
+
+
     async def get_devices_async(self) -> None:
         """ Async method to get all devices from Lupusec """
         """ This method will call the Lupusec-API to get updated data for alle devices """
@@ -355,51 +401,53 @@ class LupusecAPI:
             # Get all devices via API-CAll
             _LOGGER.debug("__init__.py.get_devices_async(): REQUEST=%s", CONST.DEVICE_LIST_REQUEST)
             get_devices_response = await LupusecAPI._async_api_call(self._ip_address, session, CONST.DEVICE_LIST_REQUEST)
-            _LOGGER.debug("...async_api_call(): done. check response...")
+            _LOGGER.debug("__init__.py.get_devices_async(): async_api_call() done. check response...")
             # Retreive Device List from Response
-            if CONST.DEVICE_LIST_HEADER in get_devices_response:
-                device_content = get_devices_response[CONST.DEVICE_LIST_HEADER]
-                print("...number of devices=", len(device_content))    
-                if (len(device_content) != 0):
-                    # empty dictionaries for various device types             
-                    binary_sensors = []
-                    switches = []
-                    covers = []
-                    # Retreive devices from device list
-                    for device in device_content:
-                        _LOGGER.debug("Device found: sid=%s, name=%s, type=%s", 
-                            device["sid"], device["name"], device["type"])
+            if get_devices_response is not None:
+                if CONST.DEVICE_LIST_HEADER in get_devices_response:
+                    device_content = get_devices_response[CONST.DEVICE_LIST_HEADER]
+                    print("...number of devices=", len(device_content))    
+                    if (len(device_content) != 0):
+                        # empty dictionaries for various device types             
+                        binary_sensors = []
+                        switches = []
+                        covers = []
+                        # Retreive devices from device list
+                        for device in device_content:
+                            _LOGGER.debug("Device found: sid=%s, name=%s, type=%s", 
+                                device["sid"], device["name"], device["type"])
 
-                        # check on device type and add to corresponding dictionary
-                        # case: binary_sensor
-                        if (device["type"] in CONST.TYPES_BIN_SENSOR):    
-                            binary_sensors.append(device)
-                            _LOGGER.debug("device is binary sensor...added.")
+                            # check on device type and add to corresponding dictionary
+                            # case: binary_sensor
+                            if (device["type"] in CONST.TYPES_BIN_SENSOR):    
+                                binary_sensors.append(device)
+                                _LOGGER.debug("device is binary sensor...added.")
 
-                        # case: switch
-                        elif (device["type"] in CONST.TYPES_SWITCH):    
-                            switches.append(device)
-                            _LOGGER.debug("device is switch...added.")
+                            # case: switch
+                            elif (device["type"] in CONST.TYPES_SWITCH):    
+                                switches.append(device)
+                                _LOGGER.debug("device is switch...added.")
 
-                        # case: cover
-                        elif (device["type"] in CONST.TYPES_COVER):    
-                            covers.append(device)
-                            _LOGGER.debug("device is cover...added.")
-                        
-                        # case: unknwon device type
-                        else :
-                            _LOGGER.debug("unknown device type...ignoring")
+                            # case: cover
+                            elif (device["type"] in CONST.TYPES_COVER):    
+                                covers.append(device)
+                                _LOGGER.debug("device is cover...added.")
+                            
+                            # case: unknwon device type
+                            else :
+                                _LOGGER.debug("unknown device type...ignoring")
 
-                    # copy results to cache dictionaries        
-                    self._cacheBinarySensors = binary_sensors
-                    self._cacheSwitches = switches
-                    self._cacheCovers = covers
-
-                else : 
-                    _LOGGER.info("ERROR: get_devices_async(): no devices found.")
-            else :
-                _LOGGER.info("ERROR: get_devices_async(): no devices found.")
+                        # copy results to cache dictionaries        
+                        self._cacheBinarySensors = binary_sensors
+                        self._cacheSwitches = switches
+                        self._cacheCovers = covers
 
+                    else : 
+                        _LOGGER.info("ERROR: get_devices_async(): no devices found.")
+                else :
+                    _LOGGER.error("ERROR: get_devices_async(): unexpected response header.")
+            else:
+                _LOGGER.error("ERROR: get_devices_async(): response is empty !")
         _LOGGER.debug("__init__.py.get_devices_async() finished.") 
 
 
@@ -421,13 +469,17 @@ class LupusecAPI:
 
             _LOGGER.debug("...API-call finished. check response...")
             # Check response 
-            if (len(self._cacheSwitches) != 0):
+            if self._cacheSwitches is None:
+                _LOGGER.error("ERROR: get_binary_sensors_async(): cacheSwitches is empty.")
+            elif (len(self._cacheSwitches) != 0):
                 _LOGGER.debug("...number of switches=%s", len(self._cacheSwitches))
             else:
                 _LOGGER.info("ERROR: get_switches_async(): no switches found.")
  
         _LOGGER.debug("get_switches_async(): checking cacheSwitches...")
-        if (len(self._cacheSwitches) != 0):
+        if self._cacheSwitches is None:
+            _LOGGER.error("ERROR: get_binary_sensors_async(): cacheSwitches is empty.")
+        elif (len(self._cacheSwitches) != 0):
             _LOGGER.debug("...number of switches=%s", len(self._cacheSwitches))
         else:
              _LOGGER.info("ERROR: get_switches_async(): no switches found.")           
@@ -452,7 +504,9 @@ class LupusecAPI:
 
             _LOGGER.debug("get_covers_async(): API-call finished. check response...")
             # Check response 
-            if (len(self._cacheCovers) != 0):
+            if self._cacheCovers is None:
+                _LOGGER.error("ERROR: get_binary_sensors_async(): cacheCovers is empty.")
+            elif (len(self._cacheCovers) != 0):
                 _LOGGER.debug("...number of covers=%s", len(self._cacheCovers))
             else:
                 _LOGGER.info("ERROR: get_covers_async(): no covers found.")
@@ -460,7 +514,9 @@ class LupusecAPI:
         _LOGGER.debug("__init__.py.get_covers_async() finished.") 
 
         _LOGGER.debug("get_covers_async(): checking cacheCovers...")
-        if (len(self._cacheCovers) != 0):
+        if self._cacheCovers is None:
+            _LOGGER.error("ERROR: get_binary_sensors_async(): cacheCovers is empty.")
+        elif (len(self._cacheCovers) != 0):
             _LOGGER.debug("...number of covers=%s", len(self._cacheCovers))
         else:
              _LOGGER.info("ERROR: get_covers_async(): no covers found.")           
@@ -485,15 +541,19 @@ class LupusecAPI:
 
             _LOGGER.debug("get_binary_sensors_async(): API-call finished. check response...")
             # Check response 
-            if (len(self._cacheBinarySensors) != 0):
+            if self._cacheBinarySensors is None:
+                _LOGGER.error("ERROR: get_binary_sensors_async(): cacheBinarySensors is empty.")
+            elif (len(self._cacheBinarySensors) != 0):
                 _LOGGER.debug("...number of binary sensors=%s", len(self._cacheBinarySensors))
             else:
-                _LOGGER.info("ERROR: get_binary_sensors_async(): no binary sensors  found.")
+                _LOGGER.error("ERROR: get_binary_sensors_async(): no binary sensors found.")
 
         _LOGGER.debug("__init__.py.get_binary_sensors_async() finished.") 
 
         _LOGGER.debug("get_binary_sensors_async(): checking cacheBinarySensors...")
-        if (len(self._cacheBinarySensors) != 0):
+        if self._cacheBinarySensors is None:
+            _LOGGER.error("ERROR: get_binary_sensors_async(): cacheBinarySensors is empty.")
+        elif (len(self._cacheBinarySensors) != 0):
             _LOGGER.debug("...number of binary sensors=%s", len(self._cacheBinarySensors))
         else:
              _LOGGER.info("ERROR: get_binary_sensors_async(): no binary sensors found.")           
@@ -502,67 +562,6 @@ class LupusecAPI:
 
 
 
-    async def get_device_async(self, device_id) -> Dict:
-        """Async method to get a single device."""
-        _LOGGER.debug("__init__.py.get_device_async() called for device_id=%s", device_id)
-        timeNow = time.time()
-        
-        # Get device from cache or update from Lupusec System
-        # ToDo: update frequency shall not be hard-coded -> transfer to constant
-        if self._cacheBinarySensors is None or timeNow - self._cacheStampP > 10.0:
-            # time elapsed, devices need update from Lupusec System.
-            if self._cacheBinarySensors is None:
-                _LOGGER.debug("__init__.py.get_device_async(): cacheBinarySensors is none, update from Lupusec System started.")
-            if timeNow - self._cacheStampP > 2.0:
-                _LOGGER.debug("__init__.py.get_device_async(): time elapsed, update from Lupusec System started.")    
-            self._cacheStampP = timeNow
-
-            async with aiohttp.ClientSession(auth=self._auth) as session:
-                _LOGGER.debug("auth.encode: %s", self._auth.encode())  
-
-                # Get all devices from LupusecAPI
-                _LOGGER.debug("__init__.py.get_device_async(): REQUEST=%s", CONST.DEVICE_LIST_REQUEST)
-                get_bin_sensors_response = await LupusecAPI._async_api_call(self._ip_address, session, CONST.DEVICE_LIST_REQUEST)
-                _LOGGER.debug("_async_api_call(): done. check response...")
-                # Retreive Device Liste from Response
-                if CONST.DEVICE_LIST_HEADER in get_bin_sensors_response:
-                    device_content = get_bin_sensors_response[CONST.DEVICE_LIST_HEADER]
-                    _LOGGER.debug("Number of devices found: %s", len(device_content))    
-                    if (len(device_content) != 0):             
-                        binary_sensors = []
-                        for device in device_content:
-                            if (device["type"] in CONST.TYPES_BIN_SENSOR):    
-                                binary_sensors.append(device)
-                                _LOGGER.debug("BinarySensor found: sid=%s, name=%s, type=%s, status=%s", 
-                                    device["sid"], device["name"], device["type"], device["status_ex"] )
-                                _LOGGER.debug("sid=%s, python_class=%s", 
-                                    device["sid"], type(device["sid"]))                                    
-                            else :
-                                _LOGGER.debug("device is no BinarySensor...skipping.")
-                        self._cacheBinarySensors = binary_sensors
-                        _LOGGER.debug("get_device_async(): No of updated BinarySensors: %s", len(self._cacheBinarySensors))
-                    else : 
-                        _LOGGER.info("ERROR: get_device_async(): no BinarySensors found.")
-                else :
-                    _LOGGER.info("ERROR: get_device_async(): no BinarySensors found.")
-            _LOGGER.debug("__init__.py.get_device_async(): update from LupusecAPI finished.") 
-
-        # time not elapsed, Devices do not need update from Lupusec System...using Cache
-        _LOGGER.debug("__init__.py.get_device_async(): using Cache to provide Update on BinarySensor.")
-        if (len(self._cacheBinarySensors) != 0):  
-            _LOGGER.debug("__init__.py.get_device_async(): Number of BinarySensors=%s", len(self._cacheBinarySensors))             
-            for sensor in self._cacheBinarySensors:
-                #_LOGGER.debug("__init__.py.get_device_async(): device with sid=%s, name=%s", sensor["sid"], sensor["name"])
-                if (sensor["sid"] == device_id):
-                    _LOGGER.debug("__init__.py.get_device_async(): matching device with sid=%s found: status=%s", sensor["sid"], sensor["status_ex"])
-                    return sensor    
-            _LOGGER.error("__init__.py.get_device_async(): %s BinarySensors checked, but no device found with sid=%s", len(self._cacheBinarySensors), device_id)
-            return {}
-        else :
-            _LOGGER.debug("ERROR: length of self._cacheBinarySensors = 0") 
-            return {}
-
-
     async def get_switch_async(self, device_id) -> Dict:
         """ Async method to get a single switch device. """
         """ device will be taken from cache or updated from API-Call """
@@ -571,9 +570,9 @@ class LupusecAPI:
         # Get device from cache or update from Lupusec System
         get_devices = await LupusecAPI.get_switches_async(self)
 
-        _LOGGER.debug("__init__.py.get_switch_async(): using Cache to provide Update on Switch.")
+        #_LOGGER.debug("__init__.py.get_switch_async(): using Cache to provide Update on Switch.")
         if (len(get_devices) != 0):  
-            _LOGGER.debug("__init__.py.get_switch_async(): Number of Switches=%s", len(get_devices))             
+            #_LOGGER.debug("__init__.py.get_switch_async(): Number of Switches=%s", len(get_devices))             
             for device in get_devices:
                 #_LOGGER.debug("__init__.py.get_switch_async(): device with sid=%s, name=%s", device["sid"], device["name"])
                 if (device["sid"] == device_id):
@@ -594,9 +593,8 @@ class LupusecAPI:
         # Get device from cache or update from Lupusec System
         get_devices = await LupusecAPI.get_covers_async(self)
 
-        _LOGGER.debug("__init__.py.get_cover_async(): using Cache to provide Update on Cover.")
         if (len(get_devices) != 0):  
-            _LOGGER.debug("__init__.py.get_cover_async(): Number of Covers=%s", len(get_devices))             
+            #_LOGGER.debug("__init__.py.get_cover_async(): Number of Covers=%s", len(get_devices))             
             for device in get_devices:
                 #_LOGGER.debug("__init__.py.get_cover_async(): device with sid=%s, name=%s", device["sid"], device["name"])
                 if (device["sid"] == device_id):
@@ -614,11 +612,11 @@ class LupusecAPI:
         _LOGGER.debug("__init__.py.get_binary_sensor_async() called for device_id=%s", device_id)
 
         # Get device from cache or update from Lupusec System
-        get_devices = await LupusecAPI.get_binary_sensor_async(self)
+        get_devices = await LupusecAPI.get_binary_sensors_async(self)
 
-        _LOGGER.debug("__init__.py.get_binary_sensor_async(): using Cache to provide Update on Binary Sensor.")
+        #_LOGGER.debug("__init__.py.get_binary_sensor_async(): using Cache to provide Update on Binary Sensor.")
         if (len(get_devices) != 0):  
-            _LOGGER.debug("__init__.py.get_binary_sensor_async(): Number of Binary Sensors=%s", len(get_devices))             
+            #_LOGGER.debug("__init__.py.get_binary_sensor_async(): Number of Binary Sensors=%s", len(get_devices))             
             for device in get_devices:
                 #_LOGGER.debug("__init__.py.get_binary_sensor_async(): device with sid=%s, name=%s", device["sid"], device["name"])
                 if (device["sid"] == device_id):
